from base64 import b64encode
from collections import namedtuple
from functools import partial
import sqlite3


sqlite3.register_converter('BLOB', b64encode)
connect = partial(sqlite3.connect, detect_types=sqlite3.PARSE_DECLTYPES)


class Req:
    def __init__(self, db):
        self.db = db

    def exec(self, req):
        self.db.cur.execute(req)

    @property
    def fields(self):
        return [c[0] for c in self.db.cur.description]

    def __call__(self):
        for row in self.db.cur.fetchall():
            yield row


class Table:
    def __init__(self, db, name):
        self.db = db
        self.name = name

    def rows(self):
        req = Req(self.db)
        req.exec(f'SELECT * FROM {self.name}')
        fields = namedtuple(self.name, req.fields)
        for row in req():
            yield fields(*row)

    def __len__(self):
        req = Req(self.db)
        req.exec(f'SELECT COUNT(*) FROM {self.name}')
        return list(req())[0][0]


class DB:
    def __init__(self, path):
        self.cnx = connect(path)
        self.cur = self.cnx.cursor()
        self._tables = []

    @property
    def tables(self):
        if not self._tables:
            req = Req(self)
            req.exec('SELECT * FROM main.sqlite_master WHERE type="table"')
            self._tables = [
                Table(self, row[1])
                for row in req()
            ]
            for table in self._tables:
                setattr(self, table.name, table)
        return self._tables

    def dictify(self):
        return {
            table.name : table
            for table in self._tables
        }

